import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:yuro_core/yuro_core.dart';

class _Translation {
  Locale locale;
  Locale fallbackLocale;
  Map<String, Map<String, String>> translations = {};
}

extension YuroTranslationExt on YuroInterface {
  static _Translation _translation = _Translation();

  Locale get locale => _translation.locale;

  set locale(Locale locale) => _translation.locale = locale;

  Locale get fallbackLocale => _translation.fallbackLocale;

  set fallbackLocale(Locale locale) => _translation.fallbackLocale = locale;

  Map<String, Map<String, String>> get translations => _translation.translations;

  void addTranslations(Map<String, Map<String, String>> translations) {
    if (translations != null) _translation.translations.addAll(translations);
  }
}

extension TranslationExt on String {
  String get tr {
    // 从主语言中查询
    final locale = Yuro.locale ?? ui.window.locale;
    if (locale?.languageCode != null) {
      if (_hitTest('${locale.languageCode}_${locale.countryCode}', this)) {
        return Yuro.translations['${locale.languageCode}_${locale.countryCode}'][this];
      } else if (_hitTest('${locale.languageCode}', this)) {
        return Yuro.translations['${locale.languageCode}'][this];
      }
    }
    // 从备用语言中查询
    final fallbackLocale = Yuro.fallbackLocale;
    if (fallbackLocale?.languageCode != null) {
      if (_hitTest('${fallbackLocale.languageCode}_${fallbackLocale.countryCode}', this)) {
        return Yuro.translations['${fallbackLocale.languageCode}_${fallbackLocale.countryCode}'][this];
      } else if (_hitTest('${fallbackLocale.languageCode}', this)) {
        return Yuro.translations['${fallbackLocale.languageCode}'][this];
      }
    }
    return this;
  }

  bool _hitTest(String localeStr, String key) {
    return Yuro.translations.containsKey(localeStr) && Yuro.translations[localeStr].containsKey(key);
  }
}
